import os
import json
import zipfile
from tqdm import tqdm
from urllib.request import urlretrieve


def calculate_intersect_area(bbox1: list, bbox2: list) -> list:
    """
    Calculates the returns the intersected area btw given bboxes.
    Bounding boxes in the form of: [xmin, ymin, xmax, ymax]
    """
    # determine the (x, y)-coordinates of the intersection rectangle
    xA = max(bbox1[0], bbox2[0])
    yA = max(bbox1[1], bbox2[1])
    xB = min(bbox1[2], bbox2[2])
    yB = min(bbox1[3], bbox2[3])

    # compute the area of intersection rectangle
    intersect_area = abs(max((xB - xA, 0)) * max((yB - yA), 0))

    return intersect_area


def get_bbox_inside_image(label_bbox: list, image_bbox: list) -> list:
    """
    Corrects label_bbox so that all points are inside image bbox.
    Returns the corrected bbox.
    """
    xA = max(label_bbox[0], image_bbox[0])
    yA = max(label_bbox[1], image_bbox[1])
    xB = min(label_bbox[2], image_bbox[2])
    yB = min(label_bbox[3], image_bbox[3])
    corrected_label_bbox = [xA, yA, xB, yB]

    return corrected_label_bbox


def list_annotation_paths_recursively(directory: str, ignore_background_only_ones: bool = True) -> list:
    """
    Accepts a folder directory containing image files.
    Returns a list of image file paths present in given directory.
    If ignore_background_only_ones is True, json's coresponding  to background
    only images are discarded.
    """

    # form image regions
    image_xmin = 0
    image_xmax = 1080
    image_ymin = 0
    image_ymax = 1920
    image_bbox = [image_xmin, image_ymin, image_xmax, image_ymax]

    # walk directories recursively and find json files
    relative_filepath_list = []
    # r=root, d=directories, f=files
    for r, _, f in os.walk(directory):
        for file in f:
            if file.split(".")[-1] in ["json"]:
                # get abs file path
                abs_filepath = os.path.join(r, file)

                # pass if sample id card json (such as 43_tur_id.json)
                if "id" in abs_filepath.split(os.sep)[-1]:
                    continue
                else:
                    try:
                        # load poly
                        with open(abs_filepath, 'r') as json_file:
                            quad = json.load(json_file)
                            coords = quad['quad']
                    except:
                        # fix for 29_irn_drvlic.json
                        continue

                # reformat corners
                label_xmin = min([pos[0] for pos in coords])
                label_xmax = max([pos[0] for pos in coords])
                label_ymin = min([pos[1] for pos in coords])
                label_ymax = max([pos[1] for pos in coords])

                # ignore label if label bbox doesnt intersect with image boox
                label_bbox = [label_xmin, label_ymin, label_xmax, label_ymax]
                if ignore_background_only_ones:
                    intersect_area = calculate_intersect_area(label_bbox, image_bbox)
                    if intersect_area < 1:
                        continue

                relative_filepath = abs_filepath.split(directory)[-1]
                relative_filepath = relative_filepath.replace("\\", "/") # for windows
                relative_filepath_list.append(relative_filepath)

    number_of_files = len(relative_filepath_list)
    folder_name = directory.split(os.sep)[-1]
    print("There are {} image files in folder {}.".format(number_of_files, folder_name))

    return relative_filepath_list


def create_dir(_dir: str):
    """
    Creates given directory if it is not present.
    """
    if not os.path.exists(_dir):
        os.makedirs(_dir)


class TqdmUpTo(tqdm):
    """
    Provides `update_to(n)` which uses `tqdm.update(delta_n)`.
    https://pypi.org/project/tqdm/#hooks-and-callbacks
    """
    def update_to(self, b=1, bsize=1, tsize=None):
        """
        b  : int, optional
            Number of blocks transferred so far [default: 1].
        bsize  : int, optional
            Size of each block (in tqdm units) [default: 1].
        tsize  : int, optional
            Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)  # will also set self.n = b * bsize


def download(url: str, save_dir: str):
    """
    Downloads file by http request, shows remaining time.
    https://pypi.org/project/tqdm/#hooks-and-callbacks
    Example inputs:
        url: 'ftp://smartengines.com/midv-500/dataset/01_alb_id.zip'
        save_dir: 'data/'
    """

    # create save_dir if not present
    create_dir(save_dir)
    # download file
    with TqdmUpTo(unit='B', unit_scale=True, miniters=1,
                  desc=url.split('/')[-1]) as t:  # all optional kwargs
        urlretrieve(url, filename=os.path.join(save_dir, url.split('/')[-1]),
                    reporthook=t.update_to, data=None)


def unzip(file_path: str, dest_dir: str):
    """
    Unzips compressed .zip file.
    Example inputs:
        file_path: 'data/01_alb_id.zip'
        dest_dir: 'data/'
    """

    # unzip file
    with zipfile.ZipFile(file_path) as zf:
        zf.extractall(dest_dir)



import unittest


class ConverterTests(unittest.TestCase):

    def test_calculate_intersect_area(self):
        from midv500.utils import calculate_intersect_area
        # check if intersection area is calculated correctly

        # case 1
        bbox1 = [25, 32, 55, 62]
        bbox2 = [26, 62, 54, 75]
        intersect_area = calculate_intersect_area(bbox1, bbox2)
        self.assertEqual(intersect_area, 0)

        # case 2
        bbox1 = [25, 32, 55, 62]
        bbox2 = [26, 30, 54, 75]
        intersect_area = calculate_intersect_area(bbox1, bbox2)
        self.assertEqual(intersect_area, 840)

    def test_get_bbox_inside_image(self):
        from midv500.utils import get_bbox_inside_image
        # check if bbox inside image is calculated correctly

        # form bbox regions
        label_bbox = [-200, 500, 2500, 600]
        # form image regions
        image_xmin = 0
        image_xmax = 1920
        image_ymin = 0
        image_ymax = 1080
        image_bbox = [image_xmin, image_ymin, image_xmax, image_ymax]
        # calculate corrected regions
        corrected_label_bbox = get_bbox_inside_image(label_bbox, image_bbox)
        self.assertEqual(corrected_label_bbox, [0, 500, 1920, 600])

    def test_list_annotation_paths_recursively(self):
        from midv500.utils import list_annotation_paths_recursively
        # check if annotations paths can be read

        directory = "tests/test_data/data/"
        # read annotations paths
        annotation_paths = list_annotation_paths_recursively(directory, ignore_background_only_ones=True)
        self.assertEqual(len(annotation_paths), 90)

    def test_download(self):
        from midv500.utils import download
        # check if annotations paths can be read

        url = "https://github.com/fcakyon/midv500/raw/master/tests/test_data/CA43_01.zip"
        save_dir = "tests/test_data/"
        # read annotations paths
        download(url, save_dir)

    def test_unzip(self):
        from midv500.utils import unzip
        # check if annotations paths can be read

        file_path = "tests/test_data/CA43_01.zip"
        dest_dir = "tests/test_data/"
        # read annotations paths
        unzip(file_path, dest_dir)


if __name__ == '__main__':
    unittest.main()



import os
import cv2
import json
import argparse
import numpy as np
from tqdm import tqdm
from midv500.utils import list_annotation_paths_recursively, get_bbox_inside_image, create_dir

def convert(root_dir: str, export_dir: str):
    """
    Walks inside root_dir (should oly contain original midv500 dataset folders),
    reads all annotations, and creates coco styled annotation file
    named as midv500_coco.json saved to export_dir.
    Sample inputs:
        root_dir: ~/data/midv500/
        export_dir: ~/data/
    """

    # raise error if export_dir is given as a json file path
    if "json" in export_dir:
        raise ValueError('export_dir should be a directory, not a file path!')

    # create export_dir if not present
    create_dir(export_dir)

    # init coco vars
    images = []
    annotations = []

    annotation_paths = list_annotation_paths_recursively(root_dir)
    print("Converting to coco.")
    for ind, rel_annotation_path in enumerate(tqdm(annotation_paths)):
        # get image path
        rel_image_path = rel_annotation_path.replace("ground_truth","images")
        rel_image_path = rel_image_path.replace("json","tif")

        # load image
        abs_image_path = os.path.join(root_dir, rel_image_path)
        image = cv2.imread(abs_image_path)
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)

        # prepare image info
        image_dict = dict()
        image_dict["file_name"] = rel_image_path
        image_dict['height'] = image.shape[0]
        image_dict['width'] = image.shape[1]
        image_dict['id'] = ind
        # add image info
        images.append(image_dict)

        # form image regions
        image_xmin = 0
        image_xmax = image.shape[1]
        image_ymin = 0
        image_ymax = image.shape[0]
        image_bbox = [image_xmin, image_ymin, image_xmax, image_ymax]

        # load mask coords
        abs_annotation_path = os.path.join(root_dir, rel_annotation_path)
        quad = json.load(open(abs_annotation_path, 'r'))
        mask_coords = quad['quad']

        # create mask from poly coords
        mask = np.zeros(image.shape, dtype=np.uint8)
        mask_coords_np = np.array(mask_coords, dtype=np.int32)
        cv2.fillPoly(mask, mask_coords_np.reshape(-1, 4, 2), color=(255,255,255))
        mask = cv2.cvtColor(mask, cv2.COLOR_BGR2GRAY)
        mask = cv2.threshold(mask, 0,255, cv2.THRESH_BINARY)[1]

        # get voc style bounding box coordinates [minx, miny, maxx, maxy] of the mask
        label_xmin = min([pos[0] for pos in mask_coords])
        label_xmax = max([pos[0] for pos in mask_coords])
        label_ymin = min([pos[1] for pos in mask_coords])
        label_ymax = max([pos[1] for pos in mask_coords])
        label_bbox = [label_xmin, label_ymin, label_xmax, label_ymax]
        label_bbox = get_bbox_inside_image(label_bbox, image_bbox)

        # calculate coco style bbox coords [minx, miny, width, height] and area
        label_area = int((label_bbox[2]-label_bbox[0]) * (label_bbox[3]-label_bbox[1]))
        label_bbox = [label_bbox[0], label_bbox[1], label_bbox[2]-label_bbox[0], label_bbox[3]-label_bbox[1]]

        # prepare annotation info
        annotation_dict = dict()
        annotation_dict["iscrowd"] = 0
        annotation_dict["image_id"] = image_dict['id']
        annotation_dict['category_id'] = 1 # id card
        annotation_dict['ignore'] = 0
        annotation_dict['id'] = ind

        annotation_dict['bbox'] = label_bbox
        annotation_dict['area'] = label_area
        annotation_dict['segmentation'] = [[single_coord for coord_pair in mask_coords for single_coord in coord_pair]]
        # add annotation info
        annotations.append(annotation_dict)

    # combine lists and form coco dict
    coco_dict = dict()
    coco_dict["images"] = images
    coco_dict["annotations"] = annotations
    coco_dict["categories"] = [{'name': 'id_card', 'id': 1}]

    # export coco dict
    export_path = os.path.join(export_dir, "midv500_coco.json")
    with open(export_path, 'w') as f:
        json.dump(coco_dict, f)


if __name__ == '__main__':
    # construct the argument parser
    ap = argparse.ArgumentParser()

    # add the arguments to the parser
    ap.add_argument("root_dir", default="data/", help="Directory of the downloaded MIDV-500 dataset.")
    ap.add_argument("export_dir", default="coco/", help="Directory for coco file to be exported.")
    args = vars(ap.parse_args())

    # convert dataset
    convert(args['root_dir'], args['export_dir'])



import os
import argparse
from midv500.utils import download, unzip

all_links = ['ftp://smartengines.com/midv-500/dataset/01_alb_id.zip',
             'ftp://smartengines.com/midv-500/dataset/02_aut_drvlic_new.zip',
             'ftp://smartengines.com/midv-500/dataset/03_aut_id_old.zip',
             'ftp://smartengines.com/midv-500/dataset/04_aut_id.zip',
             'ftp://smartengines.com/midv-500/dataset/05_aze_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/06_bra_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/07_chl_id.zip',
             'ftp://smartengines.com/midv-500/dataset/08_chn_homereturn.zip',
             'ftp://smartengines.com/midv-500/dataset/09_chn_id.zip',
             'ftp://smartengines.com/midv-500/dataset/10_cze_id.zip',
             'ftp://smartengines.com/midv-500/dataset/11_cze_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/12_deu_drvlic_new.zip',
             'ftp://smartengines.com/midv-500/dataset/13_deu_drvlic_old.zip',
             'ftp://smartengines.com/midv-500/dataset/14_deu_id_new.zip',
             'ftp://smartengines.com/midv-500/dataset/15_deu_id_old.zip',
             'ftp://smartengines.com/midv-500/dataset/16_deu_passport_new.zip',
             'ftp://smartengines.com/midv-500/dataset/17_deu_passport_old.zip',
             'ftp://smartengines.com/midv-500/dataset/18_dza_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/19_esp_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/20_esp_id_new.zip',
             'ftp://smartengines.com/midv-500/dataset/21_esp_id_old.zip',
             'ftp://smartengines.com/midv-500/dataset/22_est_id.zip',
             'ftp://smartengines.com/midv-500/dataset/23_fin_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/24_fin_id.zip',
             'ftp://smartengines.com/midv-500/dataset/25_grc_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/26_hrv_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/27_hrv_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/28_hun_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/29_irn_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/30_ita_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/31_jpn_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/32_lva_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/33_mac_id.zip',
             'ftp://smartengines.com/midv-500/dataset/34_mda_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/35_nor_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/36_pol_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/37_prt_id.zip',
             'ftp://smartengines.com/midv-500/dataset/38_rou_drvlic.zip',
             'ftp://smartengines.com/midv-500/dataset/39_rus_internalpassport.zip',
             'ftp://smartengines.com/midv-500/dataset/40_srb_id.zip',
             'ftp://smartengines.com/midv-500/dataset/41_srb_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/42_svk_id.zip',
             'ftp://smartengines.com/midv-500/dataset/43_tur_id.zip',
             'ftp://smartengines.com/midv-500/dataset/44_ukr_id.zip',
             'ftp://smartengines.com/midv-500/dataset/45_ukr_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/46_ury_passport.zip',
             'ftp://smartengines.com/midv-500/dataset/47_usa_bordercrossing.zip',
             'ftp://smartengines.com/midv-500/dataset/48_usa_passportcard.zip',
             'ftp://smartengines.com/midv-500/dataset/49_usa_ssn82.zip',
             'ftp://smartengines.com/midv-500/dataset/50_xpo_id.zip']

def download_dataset(download_dir: str):
    """
    This script downloads the MIDV-500 dataset and unzips the folders.
    """

    for link in all_links:
        print('--------------------------------------------------------------')
        # doownlaod zip file
        print('\nDownloading:', link[40:])
        download(link, download_dir)
        print('Downloaded:', link[40:])
        # unzip zip file
        print('Unzipping:', link[40:])
        zip_path = os.path.join(download_dir,link[40:])
        unzip(zip_path, download_dir)
        print('Unzipped:', link[40:].replace('.zip', ''))
        # remove zip file
        os.remove(zip_path)

if __name__ == '__main__':
    # construct the argument parser
    ap = argparse.ArgumentParser()

    # add the arguments to the parser
    ap.add_argument("download_dir", default = "data/", help="Directory for MIDV-500 dataset to be downloaded.")
    args = vars(ap.parse_args())

    # download dataset
    download_dataset(args['download_dir'])



import os
import io
import re
import setuptools


def get_long_description():
    base_dir = os.path.abspath(os.path.dirname(__file__))
    with io.open(os.path.join(base_dir, "README.md"), encoding="utf-8") as f:
        return f.read()


def get_requirements():
    with open('requirements.txt') as f:
        return f.read().splitlines()


def get_version():
    current_dir = os.path.abspath(os.path.dirname(__file__))
    version_file = os.path.join(current_dir,
                                "midv500",
                                "__init__.py")
    with io.open(version_file, encoding="utf-8") as f:
        return re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]',
                         f.read(),
                         re.M).group(1)


setuptools.setup(
    name="midv500",
    version=get_version(),
    author="Fatih Cagatay Akyon",
    author_email="fatihcagatayakyon@gmail.com",
    description="Download and convert MIDV-500 annotations to COCO instance segmentation format",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/fcakyon/midv500",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=get_requirements(),
    python_requires='>=3.5',
)
